/* -*- Mode: C; c-basic-offset:4 ; -*- */
/*
 *  (C) 2001 by Argonne National Laboratory.
 *      See COPYRIGHT in top-level directory.
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include <mpi.h>

int do_init()
{
    int data;
    int i;
    for(i=0;i<400000000;++i)
    {
        data = i + 1;
        i = data - 1;
    }
}

double f(double a)
{
    return (4.0 / (1.0 + a*a));
}

int do_io()
{
    int i;
    char *buffer;
    FILE *f;
    const int N=500;
    int len = N*N;
    buffer = malloc(sizeof(char) * len);

    for(i=0; i<N; ++i)
    {
        f = fopen("cpi.tmp", "w");
        fwrite(buffer, len, sizeof(char), f);
        fclose(f);
    }
    unlink("cpi.tmp");
}

int main(int argc,char *argv[])
{
    int    n, myid, numprocs, i;
    double PI25DT = 3.141592653589793238462643;
    double mypi, pi, h, sum, x;
    double startwtime = 0.0, endwtime;
    int    namelen;
    char   processor_name[MPI_MAX_PROCESSOR_NAME];

    MPI_Init(&argc,&argv);
    MPI_Comm_size(MPI_COMM_WORLD,&numprocs);
    MPI_Comm_rank(MPI_COMM_WORLD,&myid);
    MPI_Get_processor_name(processor_name,&namelen);

    fprintf(stdout,"Process %d of %d is on %s\n",
            myid, numprocs, processor_name);

    n = 1000000000;	/* default # of rectangles */
    n = n*5;
    h   = 1.0 / (double) n;
    sum = 0.0;
    MPI_Bcast(&n, 1, MPI_INT, 0, MPI_COMM_WORLD);

    if (myid == 0)
    {
        printf("initializing...\n");
        fflush(stdout);
        do_init(); /* unnecessary initialization for illustration purposes */
    }

    MPI_Barrier(MPI_COMM_WORLD);
    if (myid == 0)
    {
        printf("calculating pi...\n");
        fflush(stdout);
    }
    /* A slightly better approach starts from large i and works back */
#pragma omp parallel for reduction(+:sum) private(x)
    for (i = myid + 1; i <= n; i += numprocs)
    {
        x = h * ((double)i - 0.5);
        sum += f(x);
    }
    mypi = h * sum;

    if (myid == 0)
    {
        printf("collecting data...\n");
        fflush(stdout);
    }
    MPI_Reduce(&mypi, &pi, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);

    if (myid == 0) {
        printf("writing to disk...\n");
        fflush(stdout);
        do_io(); /* unnecesary i/o for illustration purposes */

        printf("pi is approximately %.16f, Error is %.16f\n",
                pi, fabs(pi - PI25DT));
        fflush(stdout);
    }

    MPI_Finalize();
    return 0;
}
