/* -*- Mode: C; c-basic-offset:4 ; -*- */
/*
 *  (C) 2001 by Argonne National Laboratory.
 *      See COPYRIGHT in top-level directory.
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>

int do_init()
{
    int data;
    int i;
    for(i=0;i<400000000;++i)
    {
        data = i + 1;
        i = data - 1;
    }
}

double f(double a)
{
    return (4.0 / (1.0 + a*a));
}

int do_io()
{
    int i;
    char *buffer;
    const int N=500;
    FILE *f;
    int len = N*N;
    buffer = malloc(sizeof(char) * len);

    for(i=0; i<N; ++i)
    {
        f = fopen("cpi.tmp", "w");
        fwrite(buffer, len, sizeof(char), f);
        fclose(f);
    }
    unlink("cpi.tmp");
}

int main(int argc,char *argv[])
{
    long long n, i;
    double PI25DT = 3.141592653589793238462643;
    double pi, h, sum, x;

    n = 1000000000;	/* default # of rectangles */
    n = n*5;
    h   = 1.0 / (double) n;
    sum = 0.0;

    printf("initializing...\n");
    do_init(); /* unnecessary initialization for illustration purposes */

    /* A slightly better approach starts from large i and works back */
    printf("calculating pi...\n");
    fflush(stdout);

#pragma omp parallel for reduction(+:sum) private(x)
    for (i = 1; i <= n; ++i)
    {
        x = h * ((double)i - 0.5);
        sum += f(x);
    }

    pi = h * sum;
    printf("pi is approximately %.16f, Error is %.16f\n",
                pi, fabs(pi - PI25DT));

    printf("writing to disk...\n");
    fflush(stdout);

    do_io(); /* unnecesary i/o for illustration purposes */
    
    printf("finished\n");
    return 0;
}
